import boto3
import requests
import json

class PENNAPI():
    def __init__(
            self,
            url = "https://api.pennsieve.io",
            api_username = "",
            api_password = "",
            aws_key = "",
            aws_secret = ""
        ):
        self.__url = url
        self.__api_username = api_username
        self.__api_password = api_password
        self.__aws_key = aws_key
        self.__aws_secret = aws_secret
        self.__api_token = None
        self.__currentdsid = None

    def set_dsid(self, dsid):
        self.__currentdsid = dsid

    def auth(self):
        #1. Get Cognito Pool config
        url = self.__url + "/authentication/cognito-config"
        r = requests.get(url)
        r.raise_for_status()
        cognito_app_client_id = r.json()["tokenPool"]["appClientId"]
        cognito_region = r.json()["region"]
        #2. Create a Cognito client
        cognito_idp_client = boto3.client(
            "cognito-idp",
            region_name=cognito_region,
            aws_access_key_id=self.__aws_key,
            aws_secret_access_key=self.__aws_secret,
        )
        #3. Authenticate
        login_response = cognito_idp_client.initiate_auth(
            AuthFlow="USER_PASSWORD_AUTH",
            AuthParameters={"USERNAME": self.__aws_key, "PASSWORD": self.__aws_secret},
            ClientId=cognito_app_client_id,
        )
        #4. Parse response
        self.__api_key = login_response["AuthenticationResult"]["AccessToken"]
        #5. Make API request
        r = requests.get(f"{self.__url}/user", headers={"Authorization": f"Bearer {self.__api_key}"})
        r.raise_for_status()
        return r.json()

    def swagger_get_datasets(self):
        url = self.__url
        url += "/datasets/?api_key="
        url += self.__api_key
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url=url, headers=headers)
        return response.json()

    def swagger_get_collections_in_set(self):
        url = self.__url
        url += "/datasets/" + str(self.__currentdsid) + "/collections"
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def swagger_get_packages_in_set(self):
        url = self.__url
        url += "/datasets/" + str(self.__currentdsid) + "/packages"
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def swagger_get_package_sources(self, pckid):
        url = self.__url
        url += "/packages/" + str(pckid) + "/sources"
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def swagger_get_package_files(self, pckid):
        url = self.__url
        url += "/packages/" + str(pckid) + "/files"
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def swagger_get_package(self,pckid):
        url = self.__url
        url += "/packages/" + str(pckid)
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def get_files_in_dataset(self):
        url = self.__url
        url += "/N:organization:618e8dd9-f8d2-4dc4-9abb-c6aaab2e78a0/"
        url += "/datasets/" + str(self.__currentdsid) + "/files"
        qstr = {"api_key":self.__api_key}
        headers = {"Accept": "*/*"}
        response = requests.request("GET", url, headers=headers, params=qstr)
        return response.json()

    def get_pckids_for_files_in_set(self):
        datadict = dict()
        resp = self.swagger_get_packages_in_set()
        packages = self.swagger_get_packages_in_set()["packages"]
        for pack in packages:
            pckid = pack["content"]["nodeId"]
            files = self.swagger_get_package_files(pckid)
            for file in files:
                filename = file["content"]["filename"]
                datadict[filename] = pckid
        return datadict

    def discover_get_datasets(self):
        url = self.__url
        url += "/discover/datasets"
        headers = {"Accept": "application/json"}
        response = requests.request("GET", url=url, headers=headers)
        return response.json()
